@@ -2,7 +2,7 @@
  * \file Rhumb.hpp
  * \brief Header for GeographicLib::Rhumb and GeographicLib::RhumbLine classes
  *
- * Copyright (c) Charles Karney (2014-2021) <charles@karney.com> and licensed
+ * Copyright (c) Charles Karney (2014-2022) <charles@karney.com> and licensed
  * under the MIT/X11 License.  For more information, see
  * https://geographiclib.sourceforge.io/
  **********************************************************************/
@@ -31,7 +31,7 @@ namespace GeographicLib {
   /**
    * \brief Solve of the direct and inverse rhumb problems.
    *
-   * The path of constant azimuth between two points on a ellipsoid at (\e
+   * The path of constant azimuth between two points on an ellipsoid at (\e
    * lat1, \e lon1) and (\e lat2, \e lon2) is called the rhumb line (also
    * called the loxodrome).  Its length is \e s12 and its azimuth is \e azi12.
    * (The azimuth is the heading measured clockwise from north.)
@@ -73,8 +73,8 @@ namespace GeographicLib {
     real _c2;
     static const int tm_maxord = GEOGRAPHICLIB_TRANSVERSEMERCATOR_ORDER;
     static const int maxpow_ = GEOGRAPHICLIB_RHUMBAREA_ORDER;
-    // _R[0] unused
-    real _R[maxpow_ + 1];
+    // _rR[0] unused
+    real _rR[maxpow_ + 1];
     static real gd(real x)
     { using std::atan; using std::sinh; return atan(sinh(x)); }
 
@@ -85,7 +85,7 @@ namespace GeographicLib {
     // See:
     //   W. M. Kahan and R. J. Fateman,
     //   Symbolic computation of divided differences,
-    //   SIGSAM Bull. 33(3), 7-28 (1999)
+    //   SIGSAM Bull. 33(2), 7-28 (1999)
     //   https://doi.org/10.1145/334714.334716
     //   http://www.cs.berkeley.edu/~fateman/papers/divdiff.pdf
 
@@ -245,7 +245,7 @@ namespace GeographicLib {
     };
 
     /**
-     * Constructor for a ellipsoid with
+     * Constructor for an ellipsoid with
      *
      * @param[in] a equatorial radius (meters).
      * @param[in] f flattening of ellipsoid.  Setting \e f = 0 gives a sphere.
@@ -428,12 +428,6 @@ namespace GeographicLib {
      *   polygon.
      **********************************************************************/
     Math::real EllipsoidArea() const { return _ell.Area(); }
-
-    /**
-     * \deprecated An old name for EquatorialRadius().
-     **********************************************************************/
-    GEOGRAPHICLIB_DEPRECATED("Use EquatorialRadius()")
-    Math::real MajorRadius() const { return EquatorialRadius(); }
     ///@}
 
     /**
@@ -466,12 +460,10 @@ namespace GeographicLib {
     typedef Math::real real;
     friend class Rhumb;
     const Rhumb& _rh;
-    bool _exact;                // TODO: RhumbLine::_exact is unused; retire
     real _lat1, _lon1, _azi12, _salp, _calp, _mu1, _psi1, _r1;
     // copy assignment not allowed
     RhumbLine& operator=(const RhumbLine&) = delete;
-    RhumbLine(const Rhumb& rh, real lat1, real lon1, real azi12,
-              bool exact);
+    RhumbLine(const Rhumb& rh, real lat1, real lon1, real azi12);
   public:
     /**
      * Construction is via default copy constructor.
