@@ -2,7 +2,7 @@
  * \file Gravity.cpp
  * \brief Command line utility for evaluating gravity fields
  *
- * Copyright (c) Charles Karney (2011-2020) <charles@karney.com> and licensed
+ * Copyright (c) Charles Karney (2011-2022) <charles@karney.com> and licensed
  * under the MIT/X11 License.  For more information, see
  * https://geographiclib.sourceforge.io/
  *
@@ -19,9 +19,9 @@
 #include <GeographicLib/Utility.hpp>
 
 #if defined(_MSC_VER)
-// Squelch warnings about constant conditional expressions and potentially
-// uninitialized local variables
-#  pragma warning (disable: 4127 4701)
+// Squelch warnings about constant conditional and enum-float expressions
+// and potentially uninitialized local variables
+#  pragma warning (disable: 4127 5055 4701)
 #endif
 
 #include "Gravity.usage"
@@ -91,13 +91,14 @@ int main(int argc, const char* const argv[]) {
       else if (arg == "-c") {
         if (m + 2 >= argc) return usage(1, true);
         try {
-          using std::abs;
+          using std::fabs;
           DMS::flag ind;
           lat = DMS::Decode(std::string(argv[++m]), ind);
           if (ind == DMS::LONGITUDE)
             throw GeographicErr("Bad hemisphere letter on latitude");
-          if (!(abs(lat) <= 90))
-            throw GeographicErr("Latitude not in [-90d, 90d]");
+          if (!(fabs(lat) <= Math::qd))
+            throw GeographicErr("Latitude not in [-" + std::to_string(Math::qd)
+                                + "d, " + std::to_string(Math::qd) + "d]");
           h = Utility::val<real>(std::string(argv[++m]));
           circle = true;
         }
@@ -295,8 +296,8 @@ int main(int argc, const char* const argv[]) {
               else
                 g.SphericalAnomaly(lat, lon, h, Dg01, xi, eta);
               Dg01 *= 100000;   // Convert to mGals
-              xi *= 3600;       // Convert to arcsecs
-              eta *= 3600;
+              xi *= Math::ds;   // Convert to arcsecs
+              eta *= Math::ds;
               *output << Utility::str(Dg01, prec) << " "
                       << Utility::str(xi, prec) << " "
                       << Utility::str(eta, prec) << eol;
